@@ -15,7 +15,7 @@ my $prefix = __PACKAGE__;
 #
 use Carp ();
 use Net::SNMP qw(oid_lex_sort);
-use Net::SNMP::Mixin::Util qw/idx2val push_error/;
+use Net::SNMP::Mixin::Util qw/idx2val push_error get_init_slot/;
 
 #
 # this module export config
@@ -108,11 +108,11 @@ Net::SNMP::Mixin::IpCidrRouteTable - mixin class for the mib-II ipCidrRouteTable
 
 =head1 VERSION
 
-Version 0.04
+Version 0.05
 
 =cut
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 =head1 SYNOPSIS
 
@@ -126,7 +126,7 @@ our $VERSION = '0.04';
   $session->mixer('Net::SNMP::Mixin::IpCidrRouteTable');
   $session->init_mixins;
   snmp_dispatcher();
-
+  $session->init_ok();
   die $session->errors if $session->errors;
 
   my @routes = $session->get_ip_cidr_route_table();
@@ -177,8 +177,11 @@ Returns a sorted list of mib-II ipCidrRouteTable entries. Every list element (ro
 
 sub get_ip_cidr_route_table {
   my $session = shift;
-  Carp::croak "'$prefix' not initialized,"
-    unless $session->{$prefix}{__initialized};
+  my $agent   = $session->hostname;
+
+  Carp::croak "$agent: '$prefix' not initialized,"
+    unless $session->init_ok($prefix);
+
 
   # stash for return values
   my @route_tbl;
@@ -261,11 +264,23 @@ Fetch the mib-II ipCidrRouteTable from the host. Don't call this method direct!
 
 =cut
 
+#
+# due to the asynchron nature, we don't know what init job is really the last, we decrement
+# the value after each callback
+#
+use constant THIS_INIT_JOBS => 1;
+
 sub _init {
   my ( $session, $reload ) = @_;
+  my $agent = $session->hostname;
 
-  die "$prefix already initalized and reload not forced.\n"
-    if $session->{$prefix}{__initialized} && not $reload;
+  die "$agent: $prefix already initialized and reload not forced.\n"
+    if exists get_init_slot($session)->{$prefix}
+    && get_init_slot($session)->{$prefix} == 0
+    && not $reload;
+
+  # set number of async init jobs for proper initialization
+  get_init_slot($session)->{$prefix} = THIS_INIT_JOBS;
 
   # populate the object with needed mib values
   #
@@ -299,11 +314,9 @@ sub _fetch_ip_cidr_route_tbl {
   );
 
   unless ( defined $result ) {
-
-    # Net::SNMP looses sometimes error messages in nonblocking
-    # mode, so we save them in an extra buffer
-    my $err_msg = $session->error;
-    push_error( $session, "$prefix: $err_msg" ) if $err_msg;
+    if (my $err_msg = $session->error) {
+      push_error($session, "$prefix: $err_msg");
+    };
     return;
   }
 
@@ -327,11 +340,9 @@ sub _ip_cidr_route_tbl_cb {
   my $vbl     = $session->var_bind_list;
 
   unless ( defined $vbl ) {
-
-    # Net::SNMP looses sometimes error messages in nonblocking
-    # mode, so we save them in an extra buffer
-    my $err_msg = $session->error;
-    push_error( $session, "$prefix: $err_msg" ) if $err_msg;
+    if (my $err_msg = $session->error) {
+      push_error($session, "$prefix: $err_msg");
+    };
     return;
   }
 
@@ -388,7 +399,10 @@ sub _ip_cidr_route_tbl_cb {
   $session->{$prefix}{ipCidrRouteStatus} =
     idx2val( $vbl, IP_CIDR_ROUTE_STATUS, undef, undef, );
 
-  $session->{$prefix}{__initialized}++;
+  # this init job is finished
+  get_init_slot($session)->{$prefix}--;
+
+  return 1;
 }
 
 unless ( caller() ) {
@@ -418,7 +432,7 @@ Karl Gaissmaier <karl.gaissmaier at uni-ulm.de>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2011 Karl Gaissmaier, all rights reserved.
+Copyright 2011-2016 Karl Gaissmaier, all rights reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
